import logging
import os

from flask import Flask

from opentelemetry import metrics
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import PeriodicExportingMetricReader
from opentelemetry.exporter.otlp.proto.grpc.metric_exporter import OTLPMetricExporter

# Lendo variáveis de ambiente
APP_NAME = os.getenv("APP_NAME", "nsj_rest_lib")
MOPE_CODE = os.getenv("MOPE_CODE")
DEFAULT_PAGE_SIZE = int(os.getenv("DEFAULT_PAGE_SIZE", 20))
USE_SQL_RETURNING_CLAUSE = (
    os.getenv("USE_SQL_RETURNING_CLAUSE", "true").lower() == "true"
)

DATABASE_HOST = os.getenv("DATABASE_HOST", "")
DATABASE_PASS = os.getenv("DATABASE_PASS", "")
DATABASE_PORT = os.getenv("DATABASE_PORT", "")
DATABASE_NAME = os.getenv("DATABASE_NAME", "")
DATABASE_USER = os.getenv("DATABASE_USER", "")
DATABASE_DRIVER = os.getenv("DATABASE_DRIVER", "POSTGRES")

DB_POOL_SIZE = int(os.getenv("DB_POOL_SIZE", 1))

CLOUD_SQL_CONN_NAME = os.getenv("CLOUD_SQL_CONN_NAME", "")
ENV = os.getenv("ENV", "")

REST_LIB_AUTO_INCREMENT_TABLE = os.getenv(
    "REST_LIB_AUTO_INCREMENT_TABLE", "seq_control"
)


def get_logger():
    return logging.getLogger(APP_NAME)


# Endpoint do OpenTelemetry Collector
OTLP_ENDPOINT = os.getenv("OTLP_ENDPOINT", "otel-collector.prometheus-otel:4317")

# Configuração do exportador OTLP
otlp_exporter = OTLPMetricExporter(endpoint=OTLP_ENDPOINT)
reader = PeriodicExportingMetricReader(otlp_exporter)
provider = MeterProvider(metric_readers=[reader])

# Define o provedor global de métricas
metrics.set_meter_provider(provider)

application = Flask("app")



import functools
from flask import request
from nsj_rest_lib.settings import get_logger, metrics
from nsj_rest_lib.util.time_grouping import TimeGrouping, get_time_grouping


class OpenTelemetryBase:
    def __init__(
        self,
        route: str,
        metric_name: str,
        counter_name: str,
        description_counter: str = "",
        tenant_field: str = "tenant",
        grupo_empresarial_field: str = "grupo_empresarial",
        time_grouping: TimeGrouping = TimeGrouping.WEEK_OF_YEAR,
        extra_fields: dict = None,
    ):
        self.route = route
        self.metric_name = metric_name
        self.counter_name = counter_name
        self.description_counter = description_counter
        self.tenant_field = tenant_field
        self.grupo_empresarial_field = grupo_empresarial_field
        self.time_grouping = time_grouping
        self.extra_fields = extra_fields or {}
        self.logger = get_logger()

    def __call__(self, func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            response = func(*args, **kwargs)
            try:
                status_code = response[1] if isinstance(response, tuple) and len(response) > 1 else ""

                metric_data = {
                    "route": self.route,
                    "method": request.method,
                    "status_code": str(status_code),
                    self.time_grouping.value: get_time_grouping(self.time_grouping.name),
                    self.tenant_field: request.headers.get(self.tenant_field, ""),
                    self.grupo_empresarial_field: request.headers.get(self.grupo_empresarial_field, ""),
                    **self.extra_fields
                }

                self._send_telemetry(metric_data)
            except Exception as e:
                self.logger.exception(f"Falha ao enviar métricas: {e}")    
            return response

        return wrapper

    def _send_telemetry(self, metric_data):
        meter = metrics.get_meter(self.metric_name)
        counter = meter.create_counter(
            name=self.counter_name,
            description=self.description_counter,
        )
        self.logger.debug(
            f"[TelemetryBase] Enviando métrica - counter: {self.counter_name} - dados: {metric_data}"
        )
        counter.add(1, metric_data)


from typing import Any

from nsj_rest_lib.descriptor.filter_operator import FilterOperator


class Filter:

    def __init__(
        self,
        operator: FilterOperator,
        value: Any,
        table_alias: str = None,
    ):
        self.operator = operator
        self.value = value
        self.table_alias = table_alias

    def __repr__(self) -> str:
        return f"{self.value}"



# Importando arquivos de configuração
import nsj_rest_lib.db_pool_config
from nsj_rest_lib.settings import application

# Configurando o healthcheck
from nsj_rest_lib.healthcheck_config import HealthCheckConfig

HealthCheckConfig(flask_application=application).config(True)

import tests.cliente_controller

if __name__ == "__main__":
    application.run(port=5000)



from nsj_rest_lib.controller.route_base import RouteBase


class CommandRoute:
    func: callable = None
    rest_lib_route = None

    def __init__(
        self,
        func: callable,
        rest_lib_route: RouteBase,
    ) -> None:
        self.func: callable = func
        self.rest_lib_route = rest_lib_route


class CommandRouter:
    # Padrão singleton
    _instance: "CommandRouter" = None

    def __init__(self) -> None:
        self._functions: dict[str, CommandRoute] = {}

    @staticmethod
    def get_instance():
        if CommandRouter._instance is None:
            CommandRouter._instance = CommandRouter()

        return CommandRouter._instance

    # Métodos de instância
    def register(self, func_name: str, func: callable, rest_lib_route: RouteBase):
        if func_name in self._functions:
            raise Exception(
                f"A função {func_name} já foi registrada anteriormente, e não pode haver registro duplicado."
            )

        self._functions[func_name] = CommandRoute(func, rest_lib_route)

    def get(self, func_name: str) -> CommandRoute:
        if func_name not in self._functions:
            raise Exception(f"Função {func_name} não registrada.")

        return self._functions[func_name]
